 /*
  * Routine to disable IP-level socket options. This code was taken from 4.4BSD
  * rlogind and kernel source, but all mistakes in it are my fault.
  *
  * Author: Wietse Venema, Eindhoven University of Technology, The Netherlands.
  * Updated code to match OpenBSD and then I added IPv6 options. Steve Grubb
  */

#ifndef lint
static char sccsid[] = "@(#) fix_options.c 1.6 97/04/08 02:29:19";
#endif

#include <sys/types.h>
#include <sys/param.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>
#include <netdb.h>
#include <stdio.h>
#include <syslog.h>

#ifndef IPOPT_OPTVAL
#define IPOPT_OPTVAL	0
#define IPOPT_OLEN	1
#endif

#include "namespace.h"
#include "tcpd.h"
#include "internal.h"

#define BUFFER_SIZE	512		/* Was: BUFSIZ */

/* fix_options - get rid of IP-level socket options 
 * This is only called by tcpd.c right now. */

void fix_options(struct request_info *request)
{
#ifdef IP_OPTIONS
    int      fd = request->fd, sslen;
    tcpd_sockaddr ss;

    /* Determine socket_type */
    sslen = sizeof(ss);
    if (getsockname(fd, &ss.sa, &sslen) < 0) {
        syslog(LOG_ERR, "getsockname: %m");
	clean_exit(request);
    }
    if (ADDR_FAM(&ss) == AF_INET) {
        struct ip_opts optbuf;
        char lbuf[BUFFER_SIZE], *lp, *cp;
        struct   protoent *ip;
        int optsize = sizeof(optbuf);
	int ipproto, i;

        if ((ip = getprotobyname("ip")) != 0)
            ipproto = ip->p_proto;
        else
            ipproto = IPPROTO_IP;

        if (getsockopt(fd, ipproto, IP_OPTIONS, (char *)&optbuf, &optsize) 
			== 0 && optsize != 0) {
            /*
	     * Properly deal with source routing entries.  The original code
	     * here was wrong.
	     */

            for (i = 0; (void *)&optbuf.ip_opts[i] - (void *)&optbuf <
	         optsize; ) {
	        u_char c = (u_char)optbuf.ip_opts[i];
		if (c == IPOPT_LSRR || c == IPOPT_SSRR) {
		    syslog(LOG_WARNING,
		      "refused connect from %s with IP source routing options",
		       eval_client(request));
                    endprotoent();
		    clean_exit(request);
		}
		if (c == IPOPT_EOL)
		    break;
		i += (c == IPOPT_NOP) ? 1 : (u_char)optbuf.ip_opts[i+1];
            }
	
	    lp = lbuf;
	    for (cp = (char *)&optbuf; (optsize > 0) && 
			    (lp < &lbuf[sizeof(lbuf)-1]);
			    cp++, optsize--, lp += 3)
	        sprintf(lp, " %2.2x", *cp);
                syslog(LOG_NOTICE,
	           "connect from %s with IP options (ignored):%s",
	           eval_client(request), lbuf);
	    if (setsockopt(fd, ipproto, IP_OPTIONS, (char *) 0, optsize) != 0) {
	        syslog(LOG_ERR, "setsockopt IP_OPTIONS NULL: %m");
	        clean_exit(request);
	    }
        }
        endprotoent();
    }
    else if (ADDR_FAM(&ss) == AF_INET6) {
#ifdef IPV6_RTHDR
        int optval, optlen;

	if (getsockopt(fd, IPPROTO_IPV6, IPV6_RTHDR, &optval, &optlen) == 0) {
	    if (optval) {
	    /*
	     *  The socket has routing instructions enabled. Need to look
	     *  at the socket a little closer.
	     *
	     *  FIXME: reject the connection for now.
	     */
                 syslog(LOG_WARNING,
		   "refused connect from %s with IPv6 routing header enabled",
		   eval_client(request));
		 clean_exit(request);
            }
	}
#endif
    }
#endif
}

