 /*
  * Routines for testing only. Not really industrial strength.
  * 
  * Author: Wietse Venema, Eindhoven University of Technology, The Netherlands.
  */

#ifndef lint
static char sccsid[] = "@(#) scaffold.c 1.6 97/03/21 19:27:24";
#endif

/* System libraries. */
#include <ctype.h>
#include <syslog.h>
#include <string.h>
#include <stdlib.h>

/* Application-specific. */
#include "namespace.h"
#include "tcpd.h"
#include "internal.h"
#include "scaffold.h"

 /*
  * These are referenced by the options module and by rfc931.c.
  */
int allow_severity = SEVERITY;
int deny_severity = LOG_WARNING;

/*
 * Verify's an address has more than numbers & dots.
 * Returns 0 if numbers & dots, 1 otherwise.
 */
static int check_hostname( const char *addr )
{
    int i;

    for (i = 0; addr[i]; ++i)
    {
        if ( !isdigit(addr[i]) && (addr[i] != '.') )
	    return 1;
    }
    return 0;
}

/* starts off a gettaddrinfo call. caller must free */
struct addrinfo *find_addr(const char *host)
{
    struct addrinfo hints, *res;
    int ret_val;

    memset(&hints, 0, sizeof(hints));
    hints.ai_flags = AI_CANONNAME;
    hints.ai_socktype = SOCK_STREAM;  /* ASSUME tcp connections */
    if (check_hostname(host) == 0)
    {
        hints.ai_family = AF_INET;
        hints.ai_flags |= AI_NUMERICHOST;
    }
    else if (strchr(host, ':'))
    {
        hints.ai_family = AF_INET6;
        hints.ai_flags |= AI_NUMERICHOST;
    }
    else
        hints.ai_family = AF_UNSPEC;

    if ((ret_val = getaddrinfo(host, NULL, &hints, &res)) < 0 ) 
    {
        tcpd_warn("can't verify address: getaddrinfo(%s) failed - %s",
	           host, gai_strerror(ret_val));
        return 0;
    }

    if( (res == NULL) || (res->ai_addr == NULL) ) 
    {
        tcpd_warn("no addresses returned");
        return 0;
    }
    return res;
}


/* check_dns - give each address thorough workout, return address count */

int check_dns(const char *host)
{
    struct request_info request;
    tcpd_sockaddr *tsin = &request.client.au;
    int     count = 0;
    struct addrinfo *res, *ressave;

    res = find_addr( host );
    if (res )
    {
        ressave = res;
   
        request_init(&request, 0);
        sock_methods(&request);
        while (res)
        {
	    memcpy((char *) tsin, res->ai_addr, res->ai_addrlen);
	    /*
	     * Force host name and address conversions. Use the request 
	     * structure as a cache. Detect hostname lookup problems. Any 
	     * name/name or name/address conflicts will be reported while 
	     * eval_hostname() does its job.
	     */
	    request_set(&request, RQ_CLIENT_ADDR, "", RQ_CLIENT_NAME, "", 0);
	    if (STR_EQ(eval_hostname(&request.client), unknown))
	        tcpd_warn("host address %s->name lookup failed",
	 	      eval_hostaddr(&request.client));
            res = res->ai_next;
	    count++;
        }

        freeaddrinfo(ressave);
    }
    return (count);
}

/* dummy function to intercept the real shell_cmd() */

/* ARGSUSED */

void shell_cmd(const char *command)
{
    if (hosts_access_verbose)
	printf("command: %s", command);
}

/* dummy function  to intercept the real clean_exit() */

/* ARGSUSED */

void clean_exit(const struct request_info *request)
{
    exit(0);
}

/* dummy function  to intercept the real rfc931() */

/* ARGSUSED */

void rfc931req(struct request_info *request)
{
    strcpy(request->user, unknown);
}

/* check_path - examine accessibility */

int check_path(const char *path, struct stat *st)
{
    struct stat stbuf;
    char    buf[BUFSIZ];

    if (stat(path, st) < 0)
	return (-1);
#ifdef notdef
    if (st->st_uid != 0)
	tcpd_warn("%s: not owned by root", path);
    if (st->st_mode & 020)
	tcpd_warn("%s: group writable", path);
#endif
    if (st->st_mode & 002)
	tcpd_warn("%s: world writable", path);
    if (path[0] == '/' && path[1] != 0 && strlen(path) < sizeof(buf)) {
	strrchr(strcpy(buf, path), '/')[0] = 0;
	(void) check_path(buf[0] ? buf : "/", &stbuf);
    }
    return (0);
}

