 /*
  * @(#) tcpd.h 1.5 96/03/19 16:22:24
  * 
  * Author: Wietse Venema, Eindhoven University of Technology, The Netherlands.
  */

#ifndef TCP_WRAPPERS_HEADER
#define TCP_WRAPPERS_HEADER

#include <netinet/in.h>
#include <netdb.h>

#define TCPD_STRING_LENGTH	256		/* hosts, users, processes */

#ifdef cplusplus
extern "C" {
#endif

/* a union to group everything for easier manipulation */
typedef union tsockaddr {
   struct sockaddr     sa;
   struct sockaddr_in  sa_in;
   struct sockaddr_in6 sa_in6;
   char                pad[128];
}tcpd_sockaddr ;
	
/* Structure to describe one communications endpoint. */
struct host_info {
    char   name[NI_MAXHOST];         /* access via eval_hostname(host) */
    char   addr[INET6_ADDRSTRLEN];   /* access via eval_hostaddr(host) */
    tcpd_sockaddr au;                /* address union or 0 */
    struct request_info *request;    /* for shared information */
    struct t_unitdata *unit;	     /* TLI transport address or 0 */
};

/* Structure to describe what we know about a service request. */
struct request_info {
    int     fd;				/* socket handle */
    char    user[TCPD_STRING_LENGTH];	/* access via eval_user(request) */
    char    daemon[TCPD_STRING_LENGTH];	/* access via eval_daemon(request) */
    char    pid[12];			/* access via eval_pid(request) */
    struct host_info client;		/* client endpoint info */
    struct host_info server;		/* server endpoint info */
    void  (*sink) (int);		/* datagram sink function or 0 */
    void  (*hostname) (struct host_info *); /* address to printable hostname */
    void  (*hostaddr) (struct host_info *); /* address to printable address */
    char  working_buf[2 * TCPD_STRING_LENGTH]; /* Thread local working buffer */
    void  (*cleanup) (struct request_info *); /* TLI cleanup function or 0 */
    struct netconfig *config;		/* TLI netdir handle */
};


/* Global functions. */

 /*
  * Routines for controlled initialization and update of request structure
  * attributes. Each attribute has its own key.
  */
	
struct request_info *request_init(struct request_info *, ...);
struct request_info *request_set(struct request_info *, ...);

 /*
  * Routines that determine whether access should be granted.
  */

int hosts_access(struct request_info *);	/* access control */
int hosts_ctl(char *dmn, char *name, char *addr, char *user);

#if defined(TLI) || defined(PTX) || defined(TLI_SEQUENT)
void fromhost(struct request_info *);   /* get/validate client host info */
#else
#define fromhost tcpd_sock_host                 /* no TLI support needed */
extern void tcpd_sock_host(struct request_info *);/* look up endpoint addr'es */
#endif

/*
 * Routines that dispose of the connection
 */
#ifdef __GNUC__
__attribute__ ((noreturn))
#endif
void refuse(struct request_info *request);
 
 
/* Global variables. */

extern int allow_severity;		/* for connection logging */
extern int deny_severity;		/* for connection logging */

enum {
	RQ_FILE = 1,	/* file descriptor */
	RQ_DAEMON,	/* server process (argv[0]) */
	RQ_USER,	/* client user name */
	RQ_CLIENT_NAME,	/* client host name */
	RQ_CLIENT_ADDR,	/* client host address */
	RQ_SERVER_NAME,	/* server host name */
	RQ_SERVER_ADDR	/* server host address */
};

#ifdef cplusplus
}
#endif 

#endif

